import { MyRedux, connect, Provider, isEqual } from "../index";
import { shallow, render, mount } from "enzyme";
import renderer from "react-test-renderer";
import React from "react";

describe("MyRedux test ", () => {
    const reducers = {
        home(state = { data: 111 }, { type, ...newState }) {
            if (type === "set_home") {
                return { ...state, ...newState };
            }
            return state;
        }
    };

    const store = new MyRedux(reducers);

    test("getState", () => {
        expect(store.getState().home).toEqual({ data: 111 });
    });

    test("dispatch", () => {
        store.dispatch({ data: 222 }, "set_home");
        expect(store.getState().home).toEqual({ data: 222 });
    });

    test("_reducerFromCache", () => {
        expect(
            store._reducerFromCache({ type: "set_home", data: 333 })
        ).toEqual(true);
    });

    test("subscribe", () => {
        const cb = jest.fn();
        store.subscribe(cb);
        store.dispatch({ data: 444 }, "set_home");
        store.dispatch({ data: 555 }, "set_home");
        expect(cb.mock.calls.length).toBe(2);
        store.dispatch({ data: 666 }, "set_home");
        expect(cb.mock.calls.length).toBe(3);
    });

    test("unsubscribe", () => {
        const cb = jest.fn();
        const unsubscribe = store.subscribe(cb);
        unsubscribe();
        store.dispatch({ data: 777 }, "set_home");
        store.dispatch({ data: 888 }, "set_home");
        expect(cb.mock.calls.length).toBe(0);
    });
    test("connect", () => {
        const App = props => <p>{props.home.data}</p>;
        const Main = connect(({ home }) => ({ home }))(App);

        const Dom = mount(
            <Provider store={store}>
                <Main />
            </Provider>
        );
        store.dispatch({ data: 999 }, "set_home");
        expect(Dom.find("p").text()).toBe("999");
    });
});

describe("util test", () => {
    const a = {
        name: 1,
        age: 2,
        hobby: [1, 2, 3]
    };
    const b = { ...a, name: 1 };
    const c = { ...a, sex: 1 };
    const d = { ...a, hobby: [1, 2, 3] };
    const e = { ...a, age: 3 };
    const f = { ...a };

    test("isEqual b", () => {
        expect(isEqual(a, b)).toBe(true);
    });
    test("isEqual c", () => {
        expect(isEqual(a, c)).toBe(false);
    });
    test("isEqual d", () => {
        expect(isEqual(a, d)).toBe(false);
    });
    test("isEqual e", () => {
        expect(isEqual(a, e)).toBe(false);
    });
    test("isEqual f", () => {
        expect(isEqual(a, f)).toBe(true);
    });
});
