import React, {
    createContext,
    Children,
    Component,
    createElement
} from "react";
const StoreContext = createContext({ store: {} });

export class Provider extends Component {
    render() {
        return (
            <StoreContext.Provider value={this.props.store}>
                {Children.only(this.props.children)}
            </StoreContext.Provider>
        );
    }
}

export class MyRedux {
    constructor(reducer, initState = {}) {
        this._(reducer, initState);
        this.dispatch();
        this._state = {
            ...this._state,
            ...initState
        };
    }

    _(reducer) {
        this._state = {};
        this._reducer = reducer;
        this._listener_arr = [];
        this._type_cache = {};
    }

    _setState(reducer, action, reducer_key, no_cache) {
        let old_state = this._state[reducer_key];
        let new_state = reducer(old_state, action);
        if (
            !no_cache &&
            reducer_key &&
            action.type &&
            old_state !== new_state
        ) {
            this._type_cache[action.type] ||
                (this._type_cache[action.type] = new Set());
            this._type_cache[action.type].add({ reducer, reducer_key });
        }
        this._state[reducer_key] = new_state;
    }

    _reducerFromCache(action) {
        let type = action.type;
        if (type && this._type_cache[type]) {
            let type_cache = this._type_cache[type];
            type_cache.forEach(args => {
                let { reducer, reducer_key } = args;
                this._setState(reducer, action, reducer_key, true);
            });
            return true;
        } else {
            return false;
        }
    }

    dispatch = (action = {}, type = false, render = true) => {
        if (typeof action === "function") {
            action(this.dispatch, this.getState);
            return this;
        }
        if (type) {
            action.type = type;
        }
        const reducers = this._reducer;

        if (!this._reducerFromCache(action)) {
            Object.keys(reducers).map(reducer_key => {
                let reducer = reducers[reducer_key];
                this._setState(reducer, action, reducer_key);
            });
        }
        if (render && this._listener_arr.length > 0) {
            this._listener_arr.reverse().map(v => {
                v();
            });
        }
        return this;
    };

    subscribe = listener => {
        if (typeof listener !== "function") {
            return false;
        }
        this._listener_arr.push(listener);
        return () => {
            if (this._listener_arr.length <= 0) {
                return;
            }
            const index = this._listener_arr.indexOf(listener);
            this._listener_arr.splice(index, 1);
        };
    };

    getState = () => {
        return this._state;
    };
}

export const connect = (mapReducer = () => {}) => component => {
    let that;
    class Wrap extends Component {
        constructor(props) {
            super();
            that = props.store;
            const reducers = { ...mapReducer(that.getState()) };
            this.state = reducers;
        }

        componentDidMount() {
            this.unsubscribe = that.subscribe(() => {
                this.setState({ ...mapReducer(that.getState()) });
            });
        }

        componentWillUnmount() {
            this.unsubscribe();
        }

        shouldComponentUpdate(nextProps, nextState) {
            if (
                isEqual(nextProps, this.props) ||
                isEqual(nextState, this.state)
            ) {
                return true;
            }
            return false;
        }

        render() {
            return createElement(component, { ...this.state });
        }
    }

    const getWrap = () => (
        <StoreContext.Consumer>
            {store => <Wrap store={store} />}
        </StoreContext.Consumer>
    );

    return getWrap;
};

export function isEqual(objA, objB) {
    if (objA === objB) {
        return true;
    }
    const keysA = Object.keys(objA);
    const keysB = Object.keys(objB);

    if (keysA.length !== keysB.length) {
        return false;
    }
    const hasOwn = Object.prototype.hasOwnProperty;
    for (let i = 0; i < keysA.length; i++) {
        if (!hasOwn.call(objB, keysA[i]) || objA[keysA[i]] !== objB[keysA[i]]) {
            return false;
        }
    }
    return true;
}
